import os
import sys

import django

app_name = "meta_edc"
tests = [
    "tests",
    "meta_ae.tests",
    "meta_consent.tests",
    "meta_dashboard.tests",
    "meta_edc.tests",
    "meta_labs.tests",
    "meta_lists.tests",
    "meta_prn.tests",
    "meta_rando.tests",
    "meta_screening.tests",
    "meta_subject.tests",
    "meta_visit_schedule.tests",
]
if __name__ == "__main__":
    os.environ["DJANGO_SETTINGS_MODULE"] = f"{app_name}.tests.test_settings"
    django.setup()
    from django.test.runner import DiscoverRunner

    tags = [t.split("=")[1] for t in sys.argv if t.startswith("--tag")]
    failfast = any([True for t in sys.argv if t.startswith("--failfast")])
    keepdb = any([True for t in sys.argv if t.startswith("--keepdb")])
    skip_checks = any([True for t in sys.argv if t.startswith("--skip_checks")])
    opts = dict(failfast=failfast, tags=tags, keepdb=keepdb, skip_checks=skip_checks)
    failures = DiscoverRunner(**opts).run_tests(tests, **opts)
    sys.exit(failures)



# Generated by Django 3.0.6 on 2020-05-24 16:44

import edc_model.models.fields.other_charfield
import edc_model.validators.date
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_prn", "0004_auto_20200403_0332"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicallosstofollowup",
            name="phone_attempts",
        ),
        migrations.RemoveField(
            model_name="losstofollowup",
            name="phone_attempts",
        ),
        migrations.AddField(
            model_name="endofstudy",
            name="ltfu_date",
            field=models.DateField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date lost to followup, if applicable",
            ),
        ),
        migrations.AddField(
            model_name="historicalendofstudy",
            name="ltfu_date",
            field=models.DateField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date lost to followup, if applicable",
            ),
        ),
        migrations.AddField(
            model_name="historicallosstofollowup",
            name="last_missed_visit_datetime",
            field=models.DateField(
                null=True, verbose_name="Date of last missed visit report submitted"
            ),
        ),
        migrations.AddField(
            model_name="historicallosstofollowup",
            name="loss_category_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicallosstofollowup",
            name="number_consecutive_missed_visits",
            field=models.DateField(
                null=True, verbose_name="Number of consecutive visits missed"
            ),
        ),
        migrations.AddField(
            model_name="losstofollowup",
            name="last_missed_visit_datetime",
            field=models.DateField(
                null=True, verbose_name="Date of last missed visit report submitted"
            ),
        ),
        migrations.AddField(
            model_name="losstofollowup",
            name="loss_category_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="losstofollowup",
            name="number_consecutive_missed_visits",
            field=models.DateField(
                null=True, verbose_name="Number of consecutive visits missed"
            ),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="loss_category",
            field=models.CharField(
                choices=[
                    ("unknown_address", "Changed to an unknown address"),
                    ("never_returned", "Did not return despite reminders"),
                    ("bad_contact_details", "Inaccurate contact details"),
                    ("OTHER", "Other, please specify ..."),
                ],
                max_length=25,
                verbose_name="Category of loss to follow up",
            ),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="loss_category",
            field=models.CharField(
                choices=[
                    ("unknown_address", "Changed to an unknown address"),
                    ("never_returned", "Did not return despite reminders"),
                    ("bad_contact_details", "Inaccurate contact details"),
                    ("OTHER", "Other, please specify ..."),
                ],
                max_length=25,
                verbose_name="Category of loss to follow up",
            ),
        ),
    ]



from django import forms
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators import INVALID_ERROR
from edc_glucose.utils import validate_glucose_as_millimoles_per_liter
from edc_lab_results.form_validator_mixins import (
    BloodResultsFormValidatorMixin,
    BloodResultsGluFormValidatorMixin,
)

from ..models import GlucoseFbg


class GlucoseFbgFormValidator(
    BloodResultsGluFormValidatorMixin, BloodResultsFormValidatorMixin, CrfFormValidator
):
    def clean(self):
        if (
            self.cleaned_data.get("report_datetime")
            and self.cleaned_data.get("assay_datetime")
            and self.cleaned_data.get("assay_datetime")
            < self.cleaned_data.get("report_datetime")
        ):
            self.raise_validation_error(
                {"assay_datetime": "Cannot be before report date"}, INVALID_ERROR
            )
        if self.cleaned_data.get("glucose_value") is not None:
            validate_glucose_as_millimoles_per_liter("glucose", self.cleaned_data)


class GlucoseFbgForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = GlucoseFbgFormValidator

    class Meta:
        model = GlucoseFbg
        fields = "__all__"



# Generated by Django 3.2.11 on 2022-07-19 21:53

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0126_auto_20220719_2142"),
    ]

    operations = [
        migrations.AddField(
            model_name="bloodresultsrft",
            name="egfr_drop_abnormal",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                null=True,
                verbose_name="abnormal",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="egfr_drop_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="egfr_drop_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="egfr_drop_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="egfr_drop_units",
            field=models.CharField(
                blank=True,
                choices=[("%", "%")],
                default="%",
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="egfr_drop_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=4,
                max_digits=10,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="eGFR Drop",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="egfr_drop_abnormal",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                null=True,
                verbose_name="abnormal",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="egfr_drop_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="egfr_drop_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="egfr_drop_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="egfr_drop_units",
            field=models.CharField(
                blank=True,
                choices=[("%", "%")],
                default="%",
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="egfr_drop_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=4,
                max_digits=10,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="eGFR Drop",
            ),
        ),
    ]



# Generated by Django 3.2.6 on 2021-09-22 17:58

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0086_auto_20210920_0229"),
    ]

    operations = [
        migrations.AlterField(
            model_name="followupexamination",
            name="hepatomegaly",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Submit SAE form. This condition often does not have any clinical signs and symptoms, it may present with an enlarge liver on examination, and symptoms of fatigue and right upper abdominal pain. The risk of developing this condition is higher in patients who are obese and who have type 2 diabetes or metabolic syndrome",
                max_length=25,
                verbose_name="Do you think the participant has hepatomegaly with steatosis?",
            ),
        ),
        migrations.AlterField(
            model_name="followupexamination",
            name="lactic_acidosis",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Submit SAE form. Classical signs of lactic acidosis include: abdominal or stomach discomfort, decreased appetite, diarrhoea, fast or shallow breathing, a general feeling of discomfort, muscle pain or cramping; and unusual sleepiness, fatigue, or weakness.",
                max_length=25,
                verbose_name="Do you think the participant has lactic acidosis?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfollowupexamination",
            name="hepatomegaly",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Submit SAE form. This condition often does not have any clinical signs and symptoms, it may present with an enlarge liver on examination, and symptoms of fatigue and right upper abdominal pain. The risk of developing this condition is higher in patients who are obese and who have type 2 diabetes or metabolic syndrome",
                max_length=25,
                verbose_name="Do you think the participant has hepatomegaly with steatosis?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfollowupexamination",
            name="lactic_acidosis",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Submit SAE form. Classical signs of lactic acidosis include: abdominal or stomach discomfort, decreased appetite, diarrhoea, fast or shallow breathing, a general feeling of discomfort, muscle pain or cramping; and unusual sleepiness, fatigue, or weakness.",
                max_length=25,
                verbose_name="Do you think the participant has lactic acidosis?",
            ),
        ),
    ]



from __future__ import annotations

from django import forms
from edc_action_item.forms import ActionItemFormMixin
from edc_form_validators import INVALID_ERROR
from edc_form_validators.form_validator import FormValidator
from edc_form_validators.form_validator_mixins import FormValidatorMixin
from edc_model_form.mixins import BaseModelFormMixin
from edc_offstudy.modelform_mixins import OffstudyNonCrfModelFormMixin
from edc_prn.modelform_mixins import PrnFormValidatorMixin
from edc_sites.forms import SiteModelFormMixin
from edc_utils.date import to_local
from edc_visit_schedule.modelform_mixins import VisitScheduleNonCrfModelFormMixin

from ..models import OffStudyMedication


class OffStudyMedicationFormValidator(PrnFormValidatorMixin, FormValidator):
    def clean(self):
        if (
            self.cleaned_data.get("stop_date")
            and self.report_datetime
            and self.cleaned_data.get("stop_date") > to_local(self.report_datetime).date()
        ):
            self.raise_validation_error(
                {"stop_date": "Cannot be after report date."}, INVALID_ERROR
            )
        if (
            self.cleaned_data.get("last_dose_date")
            and self.cleaned_data.get("stop_date")
            and self.cleaned_data.get("last_dose_date") > self.cleaned_data.get("stop_date")
        ):
            self.raise_validation_error(
                {"last_dose_date": "Cannot be after decision to stop medication."},
                INVALID_ERROR,
            )
        self.validate_other_specify(field="reason", other_specify_field="reason_other")


class OffStudyMedicationForm(
    ActionItemFormMixin,
    VisitScheduleNonCrfModelFormMixin,
    OffstudyNonCrfModelFormMixin,
    SiteModelFormMixin,
    BaseModelFormMixin,
    FormValidatorMixin,
    forms.ModelForm,
):
    form_validator_cls = OffStudyMedicationFormValidator
    get_by_model_attr = "offstudymedication_model"

    class Meta:
        model = OffStudyMedication
        fields = "__all__"
        help_text = {"subject_identifier": "(read-only)", "action_identifier": "(read-only)"}
        widgets = {
            "subject_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
            "action_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



import sys

from django.core.management import color_style
from edc_visit_schedule.site_visit_schedules import site_visit_schedules

from .phase_three import schedule, visit_schedule

style = color_style()

sys.stdout.write(style.SUCCESS("Notice: loading visit schedule for phase 3 **** \n"))
site_visit_schedules.register(visit_schedule)



from .birth_outcomes_form_validator import BirthOutcomesFormValidator
from .delivery_form_validator import DeliveryFormValidator
from .dm_followup_form_validator import DmFollowupFormValidator
from .egfr_drop_notification_form_validator import EgfrDropNotificationFormValidator
from .followup_examination_form_validator import FollowupExaminationFormValidator
from .glucose_form_validator import GlucoseFormValidator
from .health_economics_form_validator import HealthEconomicsFormValidator



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils.html import format_html
from edc_action_item.models import ActionItem
from edc_adherence.choices import MISSED_PILLS
from edc_constants.choices import YES_NO, YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE, QUESTION_RETIRED
from edc_model.models import BaseUuidModel
from edc_model_fields.fields import OtherCharField

from meta_lists.models import (
    Complications,
    DmMedications,
    DmTreatments,
    HealthcareWorkers,
    Investigations,
    MissedReferralReasons,
)

from ..constants import DM_FOLLOWUP_ACTION
from ..model_mixins import CrfWithActionModelMixin


class DmFollowup(CrfWithActionModelMixin, BaseUuidModel):

    action_name = DM_FOLLOWUP_ACTION

    referral_date = models.DateField(
        verbose_name="Date of referral to diabetes clinic",
    )

    # Diabetes clinic attendance
    attended = models.CharField(
        verbose_name=(
            "Did you attend the diabetes clinic following referral from the META Trial"
        ),
        choices=YES_NO,
        max_length=25,
    )

    missed_referral_reasons = models.ManyToManyField(
        MissedReferralReasons,
        verbose_name=(
            "If 'No', please provide a reason for not seeking further care or follow up?"
        ),
        blank=True,
    )

    other_missed_referral_reason = OtherCharField(
        verbose_name="If other 'reason for not seeking further care', please specify ...",
        null=True,
        blank=True,
    )

    not_attended_reason = models.TextField(
        verbose_name=(
            "If 'No', please provide a reason for not seeking further care or follow up?"
        ),
        default=QUESTION_RETIRED,
        editable=False,
    )

    facility_attended = models.CharField(
        verbose_name="If 'Yes', please give the name of the facility you attended",
        max_length=50,
        null=True,
        blank=True,
    )

    attended_date = models.DateField(
        verbose_name="What was the date you attended the health facility named above?",
        null=True,
        blank=True,
    )

    healthcare_workers = models.ManyToManyField(
        HealthcareWorkers,
        verbose_name="What type of health care worker did you see?",
        blank=True,
    )

    other_healthcare_workers = OtherCharField(
        verbose_name="If other 'healthcare worker', please specify ...",
        null=True,
        blank=True,
    )

    # investigations
    investigations_performed = models.CharField(
        verbose_name="Were any further investigations conducted at this visit?",
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        max_length=25,
    )

    investigations = models.ManyToManyField(
        Investigations,
        verbose_name="If 'Yes', please indicate what investigations were conducted.",
        blank=True,
    )

    other_investigations = OtherCharField(
        verbose_name="If other 'investigations', please specify ...",
        null=True,
        blank=True,
    )

    complications_checks_performed = models.CharField(
        verbose_name=(
            "Was the patient checked for diabetes related complications at this visit?"
        ),
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        max_length=25,
    )
    complications_checks = models.ManyToManyField(
        Complications,
        verbose_name="If 'Yes, which diabetes related complications were checked for?",
        blank=True,
    )

    other_complications_checks = OtherCharField(
        verbose_name=(
            "If other 'diabetes related complications checked for ', please specify ..."
        ),
        null=True,
        blank=True,
    )

    # dm treatment

    treatment_prescribed = models.CharField(
        verbose_name="Was any treatment prescribed at this visit?",
        choices=YES_NO_NA,
        max_length=25,
        default=NOT_APPLICABLE,
    )
    dm_treatments = models.ManyToManyField(
        DmTreatments,
        verbose_name="What treatment was prescribed?",
        blank=True,
    )

    on_dm_medications = models.CharField(
        verbose_name="Are you currently taking any drug therapy for diabetes?",
        choices=YES_NO_NA,
        max_length=25,
    )

    dm_medications_init_date = models.DateField(
        verbose_name="If 'Yes', please give the date when drug treatment was started.",
        null=True,
        blank=True,
    )

    dm_medications = models.ManyToManyField(
        DmMedications,
        verbose_name=(
            "If 'Yes', please indicate which diabetes drug "
            "treatments you are currently taking."
        ),
        blank=True,
    )

    other_dm_medications = OtherCharField(
        verbose_name="If other 'drug treatments', please specify ...",
        null=True,
        blank=True,
    )

    # Diabetes Medication Adherence

    medications_adherent = models.CharField(
        verbose_name="Are you taking the diabetes drug treatments regularly, i.e. every day?",
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        max_length=25,
    )

    visual_score_slider = models.CharField(
        verbose_name="Visual score",
        max_length=3,
        help_text="%",
        null=True,
        blank=True,
    )

    visual_score_confirmed = models.IntegerField(
        verbose_name=format_html(
            "<B><font color='orange'>Interviewer</font></B>: "
            "please transcribe the score indicated from above."
        ),
        validators=[MinValueValidator(0), MaxValueValidator(100)],
        help_text="%",
        null=True,
        blank=True,
    )

    last_missed_pill = models.CharField(
        verbose_name="When was the last time you missed your study pill?",
        max_length=25,
        choices=MISSED_PILLS,
        null=True,
        blank=True,
    )

    def save(self, *args, **kwargs):
        self.action_item = ActionItem.objects.get(
            subject_identifier=self.subject_identifier, action_type__name=self.action_name
        )
        self.action_identifier = self.action_item.action_identifier
        super().save(*args, **kwargs)

    class Meta(CrfWithActionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Diabetes follow-up after referral"
        verbose_name_plural = "Diabetes follow-up after referral"



from django.apps import apps as django_apps
from django.test import TestCase
from edc_sites.managers import CurrentSiteManager


class TestManagers(TestCase):
    def test_default_model_manager_not_site_manager(self):
        app_label = "meta_subject"
        app_config = django_apps.get_app_config(app_label)
        for model_cls in app_config.get_models():
            self.assertFalse(isinstance(model_cls._default_manager, (CurrentSiteManager,)))



# Generated by Django 4.2.11 on 2024-03-27 19:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0061_alter_historicalicpreferral_site_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="icpreferral",
            name="site",
        ),
        migrations.RemoveField(
            model_name="icpreferral",
            name="subject_screening",
        ),
        migrations.DeleteModel(
            name="HistoricalIcpReferral",
        ),
        migrations.DeleteModel(
            name="IcpReferral",
        ),
    ]



from django import forms
from edc_constants.constants import NO, NOT_APPLICABLE, OTHER, YES
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators import INVALID_ERROR

from ..models import SubjectVisitMissed


class SubjectVisitMissedFormValidator(CrfFormValidator):
    def clean(self):
        self.required_if(
            YES, field="contact_attempted", field_required="contact_attempts_count"
        )

        if self.cleaned_data.get("contact_made") in [NO, NOT_APPLICABLE]:
            if not self.cleaned_data.get("contact_attempts_count") and self.cleaned_data.get(
                "contact_attempts_explained"
            ):
                self.raise_validation_error(
                    {"contact_attempts_explained": "This field is not required"}, INVALID_ERROR
                )
            if (
                self.cleaned_data.get("contact_attempts_count")
                and self.cleaned_data.get("contact_attempts_count") < 3
                and not self.cleaned_data.get("contact_attempts_explained")
            ):
                self.raise_validation_error(
                    {"contact_attempts_explained": "This field is required"}, INVALID_ERROR
                )

            if (
                self.cleaned_data.get("contact_attempts_count")
                and self.cleaned_data.get("contact_attempts_count") >= 3
                and self.cleaned_data.get("contact_attempts_explained")
            ):
                self.raise_validation_error(
                    {"contact_attempts_explained": "This field is not required"}, INVALID_ERROR
                )

        self.required_if(YES, field="contact_attempted", field_required="contact_last_date")

        self.required_if(YES, field="contact_attempted", field_required="contact_made")

        self.m2m_required_if(YES, field="contact_made", m2m_field="missed_reasons")

        self.m2m_other_specify(
            OTHER, m2m_field="missed_reasons", field_other="missed_reasons_other"
        )


class SubjectVisitMissedForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = SubjectVisitMissedFormValidator

    class Meta:
        model = SubjectVisitMissed
        fields = "__all__"



def get_htn_fieldset(part=None):
    fields = [
        "htn_diagnosis",
        "on_htn_treatment",
        "htn_treatment",
        "other_htn_treatment",
        "dyslipidaemia_diagnosis",
        "on_dyslipidaemia_treatment",
        "dyslipidaemia_rx",
        "concomitant_conditions",
        "concomitant_medications",
    ]

    return (
        f"Part {part}: Hypertension",
        {"fields": tuple(fields)},
    )


def get_hiv_fieldset(part=None):
    title = "HIV, ARVs and other prophylaxis"
    if part:
        title = f"Part {part}: {title}"
    # TODO: previous_arv_regimen_start_date is PHASE 3 only
    fields = [
        "hiv_diagnosis_date",
        "arv_initiation_date",
        "viral_load",
        "viral_load_date",
        "cd4",
        "cd4_date",
        "current_arv_regimen",
        "other_current_arv_regimen",
        "current_arv_regimen_start_date",
        "has_previous_arv_regimen",
        "previous_arv_regimen",
        "other_previous_arv_regimen",
        "previous_arv_regimen_start_date",
        "on_oi_prophylaxis",
        "oi_prophylaxis",
        "other_oi_prophylaxis",
    ]
    return (
        title,
        {"fields": fields},
    )


def get_other_history_fieldset(part=None):
    return (
        f"Part {part}: Other history",
        {
            "fields": (
                "current_smoker",
                "former_smoker",
                "dm_symptoms",
                "other_dm_symptoms",
                "dm_in_family",
            )
        },
    )



# Generated by Django 4.0.5 on 2022-06-28 10:24

import django.core.validators
from django.db import migrations, models
import django_crypto_fields.fields.encrypted_char_field


class Migration(migrations.Migration):
    dependencies = [
        (
            "meta_screening",
            "0048_rename_appt_datetime_repeat_historicalscreeningpartone_repeat_appt_datetime_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="p3_ltfu",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Only applicable if the subject missed the appointment for the second stage of screening",
                max_length=15,
                verbose_name="Should additional attempts be made to contact this subject",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="p3_ltfu_comment",
            field=models.TextField(
                null=True,
                verbose_name="Provide any additional comments on this decision (or leave blank)",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="p3_ltfu_date",
            field=models.DateField(
                blank=True,
                help_text="Must be after the appointment date for the second stage of screening",
                null=True,
                verbose_name="Date decision made",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="p3_ltfu",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Only applicable if the subject missed the appointment for the second stage of screening",
                max_length=15,
                verbose_name="Should additional attempts be made to contact this subject",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="p3_ltfu_comment",
            field=models.TextField(
                null=True,
                verbose_name="Provide any additional comments on this decision (or leave blank)",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="p3_ltfu_date",
            field=models.DateField(
                blank=True,
                help_text="Must be after the appointment date for the second stage of screening",
                null=True,
                verbose_name="Date decision made",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="p3_ltfu",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Only applicable if the subject missed the appointment for the second stage of screening",
                max_length=15,
                verbose_name="Should additional attempts be made to contact this subject",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="p3_ltfu_comment",
            field=models.TextField(
                null=True,
                verbose_name="Provide any additional comments on this decision (or leave blank)",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="p3_ltfu_date",
            field=models.DateField(
                blank=True,
                help_text="Must be after the appointment date for the second stage of screening",
                null=True,
                verbose_name="Date decision made",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="p3_ltfu",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Only applicable if the subject missed the appointment for the second stage of screening",
                max_length=15,
                verbose_name="Should additional attempts be made to contact this subject",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="p3_ltfu_comment",
            field=models.TextField(
                null=True,
                verbose_name="Provide any additional comments on this decision (or leave blank)",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="p3_ltfu_date",
            field=models.DateField(
                blank=True,
                help_text="Must be after the appointment date for the second stage of screening",
                null=True,
                verbose_name="Date decision made",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="p3_ltfu",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Only applicable if the subject missed the appointment for the second stage of screening",
                max_length=15,
                verbose_name="Should additional attempts be made to contact this subject",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="p3_ltfu_comment",
            field=models.TextField(
                null=True,
                verbose_name="Provide any additional comments on this decision (or leave blank)",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="p3_ltfu_date",
            field=models.DateField(
                blank=True,
                help_text="Must be after the appointment date for the second stage of screening",
                null=True,
                verbose_name="Date decision made",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="contact_number",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Provide a contact number if repeating glucose measures (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^[0-9\\-\\(\\)\\ ]+$", message="Enter a valid number"
                    )
                ],
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="contact_number",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Provide a contact number if repeating glucose measures (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^[0-9\\-\\(\\)\\ ]+$", message="Enter a valid number"
                    )
                ],
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="contact_number",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Provide a contact number if repeating glucose measures (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^[0-9\\-\\(\\)\\ ]+$", message="Enter a valid number"
                    )
                ],
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="contact_number",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Provide a contact number if repeating glucose measures (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^[0-9\\-\\(\\)\\ ]+$", message="Enter a valid number"
                    )
                ],
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="contact_number",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Provide a contact number if repeating glucose measures (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^[0-9\\-\\(\\)\\ ]+$", message="Enter a valid number"
                    )
                ],
            ),
        ),
    ]



# Generated by Django 5.0.1 on 2024-02-12 22:37

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "meta_lists",
            "0016_alter_abnormalfootappearanceobservations_options_and_more",
        ),
    ]

    operations = [
        migrations.CreateModel(
            name="Complications",
            fields=[
                (
                    "name",
                    models.CharField(
                        help_text="This is the stored value, required",
                        max_length=250,
                        unique=True,
                        verbose_name="Stored value",
                    ),
                ),
                (
                    "plural_name",
                    models.CharField(max_length=250, null=True, verbose_name="Plural name"),
                ),
                (
                    "display_name",
                    models.CharField(
                        help_text="(suggest 40 characters max.)",
                        max_length=250,
                        unique=True,
                        verbose_name="Name",
                    ),
                ),
                (
                    "display_index",
                    models.IntegerField(
                        default=0,
                        help_text="Index to control display order if not alphabetical, not required",
                        verbose_name="display index",
                    ),
                ),
                (
                    "field_name",
                    models.CharField(
                        blank=True,
                        editable=False,
                        help_text="Not required",
                        max_length=25,
                        null=True,
                    ),
                ),
                ("extra_value", models.CharField(max_length=250, null=True)),
                (
                    "version",
                    models.CharField(default="1.0", editable=False, max_length=35),
                ),
                ("id", models.AutoField(primary_key=True, serialize=False)),
            ],
            options={
                "verbose_name": "Complications",
                "verbose_name_plural": "Complications",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "indexes": [
                    models.Index(
                        fields=["display_index", "display_name"],
                        name="meta_lists__display_53d587_idx",
                    )
                ],
            },
        ),
        migrations.CreateModel(
            name="DmMedications",
            fields=[
                (
                    "name",
                    models.CharField(
                        help_text="This is the stored value, required",
                        max_length=250,
                        unique=True,
                        verbose_name="Stored value",
                    ),
                ),
                (
                    "plural_name",
                    models.CharField(max_length=250, null=True, verbose_name="Plural name"),
                ),
                (
                    "display_name",
                    models.CharField(
                        help_text="(suggest 40 characters max.)",
                        max_length=250,
                        unique=True,
                        verbose_name="Name",
                    ),
                ),
                (
                    "display_index",
                    models.IntegerField(
                        default=0,
                        help_text="Index to control display order if not alphabetical, not required",
                        verbose_name="display index",
                    ),
                ),
                (
                    "field_name",
                    models.CharField(
                        blank=True,
                        editable=False,
                        help_text="Not required",
                        max_length=25,
                        null=True,
                    ),
                ),
                ("extra_value", models.CharField(max_length=250, null=True)),
                (
                    "version",
                    models.CharField(default="1.0", editable=False, max_length=35),
                ),
                ("id", models.AutoField(primary_key=True, serialize=False)),
            ],
            options={
                "verbose_name": "Diabetes Medications",
                "verbose_name_plural": "Diabetes Medications",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "indexes": [
                    models.Index(
                        fields=["display_index", "display_name"],
                        name="meta_lists__display_7cc011_idx",
                    )
                ],
            },
        ),
        migrations.CreateModel(
            name="DmTreatments",
            fields=[
                (
                    "name",
                    models.CharField(
                        help_text="This is the stored value, required",
                        max_length=250,
                        unique=True,
                        verbose_name="Stored value",
                    ),
                ),
                (
                    "plural_name",
                    models.CharField(max_length=250, null=True, verbose_name="Plural name"),
                ),
                (
                    "display_name",
                    models.CharField(
                        help_text="(suggest 40 characters max.)",
                        max_length=250,
                        unique=True,
                        verbose_name="Name",
                    ),
                ),
                (
                    "display_index",
                    models.IntegerField(
                        default=0,
                        help_text="Index to control display order if not alphabetical, not required",
                        verbose_name="display index",
                    ),
                ),
                (
                    "field_name",
                    models.CharField(
                        blank=True,
                        editable=False,
                        help_text="Not required",
                        max_length=25,
                        null=True,
                    ),
                ),
                ("extra_value", models.CharField(max_length=250, null=True)),
                (
                    "version",
                    models.CharField(default="1.0", editable=False, max_length=35),
                ),
                ("id", models.AutoField(primary_key=True, serialize=False)),
            ],
            options={
                "verbose_name": "DmTreatments",
                "verbose_name_plural": "DmTreatments",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "indexes": [
                    models.Index(
                        fields=["display_index", "display_name"],
                        name="meta_lists__display_676b56_idx",
                    )
                ],
            },
        ),
        migrations.CreateModel(
            name="HealthcareWorkers",
            fields=[
                (
                    "name",
                    models.CharField(
                        help_text="This is the stored value, required",
                        max_length=250,
                        unique=True,
                        verbose_name="Stored value",
                    ),
                ),
                (
                    "plural_name",
                    models.CharField(max_length=250, null=True, verbose_name="Plural name"),
                ),
                (
                    "display_name",
                    models.CharField(
                        help_text="(suggest 40 characters max.)",
                        max_length=250,
                        unique=True,
                        verbose_name="Name",
                    ),
                ),
                (
                    "display_index",
                    models.IntegerField(
                        default=0,
                        help_text="Index to control display order if not alphabetical, not required",
                        verbose_name="display index",
                    ),
                ),
                (
                    "field_name",
                    models.CharField(
                        blank=True,
                        editable=False,
                        help_text="Not required",
                        max_length=25,
                        null=True,
                    ),
                ),
                ("extra_value", models.CharField(max_length=250, null=True)),
                (
                    "version",
                    models.CharField(default="1.0", editable=False, max_length=35),
                ),
                ("id", models.AutoField(primary_key=True, serialize=False)),
            ],
            options={
                "verbose_name": "Healthcare worker",
                "verbose_name_plural": "Healthcare workers",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "indexes": [
                    models.Index(
                        fields=["display_index", "display_name"],
                        name="meta_lists__display_2a312f_idx",
                    )
                ],
            },
        ),
        migrations.CreateModel(
            name="Investigations",
            fields=[
                (
                    "name",
                    models.CharField(
                        help_text="This is the stored value, required",
                        max_length=250,
                        unique=True,
                        verbose_name="Stored value",
                    ),
                ),
                (
                    "plural_name",
                    models.CharField(max_length=250, null=True, verbose_name="Plural name"),
                ),
                (
                    "display_name",
                    models.CharField(
                        help_text="(suggest 40 characters max.)",
                        max_length=250,
                        unique=True,
                        verbose_name="Name",
                    ),
                ),
                (
                    "display_index",
                    models.IntegerField(
                        default=0,
                        help_text="Index to control display order if not alphabetical, not required",
                        verbose_name="display index",
                    ),
                ),
                (
                    "field_name",
                    models.CharField(
                        blank=True,
                        editable=False,
                        help_text="Not required",
                        max_length=25,
                        null=True,
                    ),
                ),
                ("extra_value", models.CharField(max_length=250, null=True)),
                (
                    "version",
                    models.CharField(default="1.0", editable=False, max_length=35),
                ),
                ("id", models.AutoField(primary_key=True, serialize=False)),
            ],
            options={
                "verbose_name": "Investigations",
                "verbose_name_plural": "Investigations",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "indexes": [
                    models.Index(
                        fields=["display_index", "display_name"],
                        name="meta_lists__display_85d254_idx",
                    )
                ],
            },
        ),
    ]



from __future__ import absolute_import, unicode_literals

import os

from celery import Celery
from django.conf import settings

# set the default Django settings module for the 'celery' program.
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "meta_edc.settings")

app = Celery("meta_edc")

# Using a string here means the worker doesn't have to serialize
# the configuration object to child processes.
# - namespace='CELERY' means all celery-related configuration keys
#   should have a `CELERY_` prefix.
app.config_from_object("django.conf:settings", namespace="CELERY")

# Load task modules from all registered Django app configs.
app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)


@app.task(bind=True)
def debug_task(self):
    print("Request: {0!r}".format(self.request))



from edc_lab_panel.panels import (
    fbc_panel,
    hba1c_panel,
    insulin_panel,
    lft_panel,
    lipids_panel,
    rft_panel,
)
from edc_visit_schedule.visit import Requisition, RequisitionCollection

requisitions_prn = RequisitionCollection(
    Requisition(show_order=220, panel=hba1c_panel, required=True, additional=False),
    Requisition(show_order=230, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=240, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=250, panel=fbc_panel, required=True, additional=False),
    Requisition(show_order=260, panel=lipids_panel, required=True, additional=False),
    Requisition(show_order=270, panel=insulin_panel, required=True, additional=False),
    name="requisitions_prn",
)

requisitions_unscheduled = RequisitionCollection(name="requisitions_unscheduled")

requisitions_d1 = RequisitionCollection(
    Requisition(show_order=30, panel=fbc_panel, required=True, additional=False),
    Requisition(show_order=40, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=50, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=60, panel=insulin_panel, required=True, additional=False),
    name="requisitions_day1",
)

requisitions_w2 = RequisitionCollection(name="requisitions_week2")

requisitions_1m = RequisitionCollection(name="requisitions_month1")

requisitions_3m = RequisitionCollection(
    Requisition(show_order=20, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=30, panel=lft_panel, required=True, additional=False),
    name="requisitions_month3",
)

requisitions_6m = RequisitionCollection(
    Requisition(show_order=20, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=30, panel=lft_panel, required=True, additional=False),
    name="requisitions_month6",
)

requisitions_9m = RequisitionCollection(name="requisitions_month9")

requisitions_12m = RequisitionCollection(
    Requisition(show_order=30, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=40, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=50, panel=lipids_panel, required=True, additional=False),
    Requisition(show_order=60, panel=fbc_panel, required=True, additional=False),
    name="requisitions_month12",
)

requisitions_15m = RequisitionCollection(name="requisitions_month15")

requisitions_18m = RequisitionCollection(
    Requisition(show_order=20, panel=fbc_panel, required=True, additional=False),
    name="requisitions_month18",
)

requisitions_21m = RequisitionCollection(name="requisitions_month21")

requisitions_24m = RequisitionCollection(
    Requisition(show_order=30, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=40, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=50, panel=lipids_panel, required=True, additional=False),
    Requisition(show_order=60, panel=fbc_panel, required=True, additional=False),
    name="requisitions_month24",
)

requisitions_27m = RequisitionCollection(name="requisitions_month27")

requisitions_30m = RequisitionCollection(
    Requisition(show_order=20, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=30, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=40, panel=fbc_panel, required=True, additional=False),
    name="requisitions_month30",
)

requisitions_33m = RequisitionCollection(name="requisitions_month33")

requisitions_36m = RequisitionCollection(
    Requisition(show_order=30, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=40, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=50, panel=lipids_panel, required=True, additional=False),
    Requisition(show_order=60, panel=fbc_panel, required=True, additional=False),
    name="requisitions_month36",
)



from __future__ import annotations

from dataclasses import dataclass, field
from typing import Type

from django.utils.translation import gettext as _
from edc_constants.constants import TBD, YES
from edc_subject_dashboard.view_utils import ADD, CHANGE, VIEW
from edc_subject_dashboard.view_utils.subject_screening_button import (
    SubjectScreeningButton as BaseSubjectScreeningButton,
)

from meta_screening.models import ScreeningPartOne, ScreeningPartThree, ScreeningPartTwo

__all__ = [
    "SubjectScreeningPartOneButton",
    "SubjectScreeningPartTwoButton",
    "SubjectScreeningPartThreeButton",
]


@dataclass
class SubjectScreeningButton(BaseSubjectScreeningButton):
    next_url_name: str = field(default="screening_listboard_url")

    def __post_init__(self):
        self.model_obj = self.model_cls.objects.get(
            screening_identifier=self.model_obj.screening_identifier
        )

    @property
    def label(self) -> str:
        return "P1"

    @property
    def color(self) -> str:
        return self.colors[self.action]

    @property
    def title(self) -> str:
        if self.perms.view_only or self.model_obj.consented:
            title = _("View")
        else:
            title = _("Edit")
        return title


@dataclass
class SubjectScreeningPartOneButton(SubjectScreeningButton):
    model_obj: ScreeningPartOne = None
    model_cls: Type[ScreeningPartOne] = field(default=ScreeningPartOne)

    @property
    def label(self) -> str:
        return "P1"


@dataclass
class SubjectScreeningPartTwoButton(SubjectScreeningButton):
    model_obj: ScreeningPartTwo = None
    model_cls: Type[ScreeningPartTwo] = field(default=ScreeningPartTwo)

    @property
    def label(self) -> str:
        return "P2"

    @property
    def color(self) -> str:
        if self.action == ADD and self.disabled:
            return "default"
        return self.colors[self.action]

    @property
    def action(self):
        if not self._action:
            self._action = VIEW
            if self.model_obj.eligible_part_two == TBD:
                self._action = ADD
            elif self.model_obj:
                if self.perms.change:
                    self._action = CHANGE
        return self._action

    @property
    def disabled(self) -> str:
        if self.model_obj.eligible_part_one == YES and self.model_obj.continue_part_two == YES:
            return ""
        return "disabled"


@dataclass
class SubjectScreeningPartThreeButton(SubjectScreeningButton):
    model_obj: ScreeningPartThree = None
    model_cls: Type[ScreeningPartThree] = field(default=ScreeningPartThree)

    @property
    def label(self) -> str:
        return "P3"

    @property
    def color(self) -> str:
        if self.action == ADD and self.disabled:
            return "default"
        return self.colors[self.action]

    @property
    def action(self):
        if not self._action:
            self._action = VIEW
            if self.model_obj.eligible_part_three == TBD:
                self._action = ADD
            elif self.model_obj:
                if self.perms.change:
                    self._action = CHANGE
        return self._action

    @property
    def disabled(self) -> str:
        if self.model_obj.eligible_part_one == YES and self.model_obj.eligible_part_two == YES:
            return ""
        return "disabled"



# Generated by Django 3.2.11 on 2022-04-14 21:43

from django.db import migrations
from django.db.migrations import RunPython
from edc_consent.utils import get_consent_model_name
from edc_pharmacy.prescribe import create_prescription

from meta_edc.meta_version import get_meta_version
from meta_pharmacy.constants import METFORMIN


def func(apps, schema_editor):
    medication = apps.get_model("edc_pharmacy.medication")._default_manager.get(name=METFORMIN)
    site_model_cls = apps.get_model("sites.site")
    subject_identifiers = [
        tpl[0]
        for tpl in apps.get_model("edc_pharmacy.rx")._default_manager.values_list(
            "subject_identifier"
        )
    ]
    for subject_consent in apps.get_model(get_consent_model_name())._default_manager.exclude(
        subject_identifier__in=subject_identifiers
    ):
        site = site_model_cls.objects.get(id=subject_consent.site.id)
        create_prescription(
            subject_identifier=subject_consent.subject_identifier,
            report_datetime=subject_consent.consent_datetime,
            randomizer_name=get_meta_version(),
            medications=[METFORMIN],
            site_id=site.id,
        )
    for obj in apps.get_model("edc_pharmacy.rx")._default_manager.all():
        obj.medications.clear()
        obj.medications.add(medication)


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0106_auto_20220414_1741"),
    ]

    operations = [RunPython(func)]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import ActionItemModelAdminMixin, action_fieldset_tuple
from edc_adherence.model_admin_mixin import get_visual_score_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_subject_admin
from ..forms import DmFollowupForm
from ..models import DmFollowup
from .modeladmin import CrfModelAdminMixin


@admin.register(DmFollowup, site=meta_subject_admin)
class DmFollowupAdmin(
    CrfModelAdminMixin,
    FormLabelModelAdminMixin,
    ActionItemModelAdminMixin,
    SimpleHistoryAdmin,
):
    form = DmFollowupForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        ("Referral to Diabetes clinic", {"fields": ("referral_date",)}),
        (
            "Diabetes clinic attendance",
            {
                "fields": (
                    "attended",
                    "facility_attended",
                    "missed_referral_reasons",
                    "other_missed_referral_reason",
                    "attended_date",
                    "healthcare_workers",
                    "other_healthcare_workers",
                ),
            },
        ),
        (
            "Investigations",
            {
                "fields": (
                    "investigations_performed",
                    "investigations",
                    "other_investigations",
                    "complications_checks",
                ),
            },
        ),
        (
            "Diabetes treatment",
            {
                "fields": (
                    "treatment_prescribed",
                    "dm_treatments",
                    "on_dm_medications",
                    "dm_medications_init_date",
                    "dm_medications",
                    "other_dm_medications",
                ),
            },
        ),
        (
            "Diabetes Medication Adherence",
            {
                "fields": (
                    "medications_adherent",
                    "last_missed_pill",
                ),
            },
        ),
        get_visual_score_fieldset_tuple(),
        crf_status_fieldset,
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    filter_horizontal = (
        "healthcare_workers",
        "investigations",
        "complications_checks",
        "dm_treatments",
        "dm_medications",
        "missed_referral_reasons",
    )

    radio_fields = {
        "attended": admin.VERTICAL,
        "investigations_performed": admin.VERTICAL,
        "on_dm_medications": admin.VERTICAL,
        "medications_adherent": admin.VERTICAL,
        "last_missed_pill": admin.VERTICAL,
        "treatment_prescribed": admin.VERTICAL,
    }



# Generated by Django 3.2.11 on 2022-03-04 19:33

import django.core.validators
import edc_model.validators.date
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0034_auto_20220304_0508"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalscreeningpartone",
            name="ogtt2_performed",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartthree",
            name="ogtt2_performed",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningparttwo",
            name="ogtt2_performed",
        ),
        migrations.RemoveField(
            model_name="historicalsubjectscreening",
            name="ogtt2_performed",
        ),
        migrations.RemoveField(
            model_name="subjectscreening",
            name="ogtt2_performed",
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="converted_ifg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT Fasting glucose",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT Blood glucose level 2-hours",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="ifg2_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="<u>Time</u> FBG level measured"
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="ifg2_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
                verbose_name="FBG quantifier",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="ifg2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="IFG units",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="ifg2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="repeat_fasting",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="As reported by patient",
                max_length=15,
                null=True,
                verbose_name="Has the participant fasted?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="repeat_fasting_duration_minutes",
            field=models.IntegerField(help_text="system calculated value", null=True),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="repeat_fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Expected something like 1h20m, 11h5m, etc",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="repeat_fasting_opinion",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No"), ("not_sure", "Not sure")],
                max_length=15,
                null=True,
                verbose_name="In the opinion of the clinican, has the participant fasted for these repeat glucose measurement?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="repeat_glucose_opinion",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                help_text="If to be repeated, do so at least 3 days after the first OGTT",
                max_length=15,
                verbose_name="In opinion of the clinician, should the glucose measurements be repeated?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="repeat_glucose_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("PENDING", "Pending")],
                default="No",
                help_text="If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)",
                max_length=15,
                verbose_name="Were the glucose measurements repeated?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="converted_ifg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT Fasting glucose",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT Blood glucose level 2-hours",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="ifg2_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="<u>Time</u> FBG level measured"
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="ifg2_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
                verbose_name="FBG quantifier",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="ifg2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="IFG units",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="ifg2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="repeat_fasting",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="As reported by patient",
                max_length=15,
                null=True,
                verbose_name="Has the participant fasted?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="repeat_fasting_duration_minutes",
            field=models.IntegerField(help_text="system calculated value", null=True),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="repeat_fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Expected something like 1h20m, 11h5m, etc",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="repeat_fasting_opinion",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No"), ("not_sure", "Not sure")],
                max_length=15,
                null=True,
                verbose_name="In the opinion of the clinican, has the participant fasted for these repeat glucose measurement?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="repeat_glucose_opinion",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                help_text="If to be repeated, do so at least 3 days after the first OGTT",
                max_length=15,
                verbose_name="In opinion of the clinician, should the glucose measurements be repeated?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="repeat_glucose_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("PENDING", "Pending")],
                default="No",
                help_text="If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)",
                max_length=15,
                verbose_name="Were the glucose measurements repeated?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="converted_ifg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT Fasting glucose",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT Blood glucose level 2-hours",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="ifg2_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="<u>Time</u> FBG level measured"
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="ifg2_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
                verbose_name="FBG quantifier",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="ifg2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="IFG units",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="ifg2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="repeat_fasting",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="As reported by patient",
                max_length=15,
                null=True,
                verbose_name="Has the participant fasted?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="repeat_fasting_duration_minutes",
            field=models.IntegerField(help_text="system calculated value", null=True),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="repeat_fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Expected something like 1h20m, 11h5m, etc",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="repeat_fasting_opinion",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No"), ("not_sure", "Not sure")],
                max_length=15,
                null=True,
                verbose_name="In the opinion of the clinican, has the participant fasted for these repeat glucose measurement?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="repeat_glucose_opinion",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                help_text="If to be repeated, do so at least 3 days after the first OGTT",
                max_length=15,
                verbose_name="In opinion of the clinician, should the glucose measurements be repeated?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="repeat_glucose_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("PENDING", "Pending")],
                default="No",
                help_text="If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)",
                max_length=15,
                verbose_name="Were the glucose measurements repeated?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="converted_ifg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT Fasting glucose",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT Blood glucose level 2-hours",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="ifg2_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="<u>Time</u> FBG level measured"
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="ifg2_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
                verbose_name="FBG quantifier",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="ifg2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="IFG units",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="ifg2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="repeat_fasting",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="As reported by patient",
                max_length=15,
                null=True,
                verbose_name="Has the participant fasted?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="repeat_fasting_duration_minutes",
            field=models.IntegerField(help_text="system calculated value", null=True),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="repeat_fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Expected something like 1h20m, 11h5m, etc",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="repeat_fasting_opinion",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No"), ("not_sure", "Not sure")],
                max_length=15,
                null=True,
                verbose_name="In the opinion of the clinican, has the participant fasted for these repeat glucose measurement?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="repeat_glucose_opinion",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                help_text="If to be repeated, do so at least 3 days after the first OGTT",
                max_length=15,
                verbose_name="In opinion of the clinician, should the glucose measurements be repeated?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="repeat_glucose_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("PENDING", "Pending")],
                default="No",
                help_text="If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)",
                max_length=15,
                verbose_name="Were the glucose measurements repeated?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="converted_ifg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT Fasting glucose",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT Blood glucose level 2-hours",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="ifg2_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="<u>Time</u> FBG level measured"
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="ifg2_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
                verbose_name="FBG quantifier",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="ifg2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="IFG units",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="ifg2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="repeat_fasting",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="As reported by patient",
                max_length=15,
                null=True,
                verbose_name="Has the participant fasted?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="repeat_fasting_duration_minutes",
            field=models.IntegerField(help_text="system calculated value", null=True),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="repeat_fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Expected something like 1h20m, 11h5m, etc",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="repeat_fasting_opinion",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No"), ("not_sure", "Not sure")],
                max_length=15,
                null=True,
                verbose_name="In the opinion of the clinican, has the participant fasted for these repeat glucose measurement?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="repeat_glucose_opinion",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                help_text="If to be repeated, do so at least 3 days after the first OGTT",
                max_length=15,
                verbose_name="In opinion of the clinician, should the glucose measurements be repeated?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="repeat_glucose_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("PENDING", "Pending")],
                default="No",
                help_text="If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)",
                max_length=15,
                verbose_name="Were the glucose measurements repeated?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
    ]



# Generated by Django 5.0.1 on 2024-02-14 03:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0172_remove_historicalbloodresultsglu_action_item_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="glucosefbg",
            name="action_identifier",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="action_item",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="action_item_reason",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="missing",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="missing_count",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="parent_action_identifier",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="parent_action_item",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="related_action_identifier",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="related_action_item",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="requisition",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="results_abnormal",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="results_reportable",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="summary",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="action_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="action_item",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="action_item_reason",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="missing",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="missing_count",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="parent_action_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="parent_action_item",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="related_action_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="related_action_item",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="requisition",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="results_abnormal",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="results_reportable",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="summary",
        ),
    ]



# Generated by Django 3.2.4 on 2021-06-23 23:25

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_lists", "0010_auto_20200617_1738"),
    ]

    operations = [
        migrations.AlterField(
            model_name="arvregimens",
            name="id",
            field=models.AutoField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name="baselinesymptoms",
            name="id",
            field=models.AutoField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name="diabetessymptoms",
            name="id",
            field=models.AutoField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name="hypertensionmedications",
            name="id",
            field=models.AutoField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name="nonadherencereasons",
            name="id",
            field=models.AutoField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name="offstudyreasons",
            name="id",
            field=models.AutoField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name="oiprophylaxis",
            name="id",
            field=models.AutoField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name="subjectvisitmissedreasons",
            name="id",
            field=models.AutoField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name="symptoms",
            name="id",
            field=models.AutoField(primary_key=True, serialize=False),
        ),
    ]



from edc_microscopy.model_mixins import MalariaTestModelMixin
from edc_model.models import BaseUuidModel

from ..model_mixins import CrfModelMixin


class MalariaTest(MalariaTestModelMixin, CrfModelMixin, BaseUuidModel):
    class Meta(MalariaTestModelMixin.Meta, CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Malaria Test"



# Generated by Django 3.2.6 on 2021-09-10 16:06

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_protocol_incident", "0001_initial"),
        ("meta_prn", "0009_auto_20210910_0239"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalprotocoldeviationviolation",
            old_name="action_required",
            new_name="action_required_old",
        ),
        migrations.RenameField(
            model_name="protocoldeviationviolation",
            old_name="action_required",
            new_name="action_required_old",
        ),
        migrations.AlterField(
            model_name="historicalprotocoldeviationviolation",
            name="action_required_old",
            field=models.CharField(
                choices=[
                    ("remain_on_study", "Participant to remain on trial"),
                    ("to_be_withdrawn", "Participant to be withdrawn from trial"),
                    (
                        "remain_on_study_modified",
                        "Patient remains on study but data analysis will be modified",
                    ),
                ],
                max_length=45,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="protocoldeviationviolation",
            name="action_required_old",
            field=models.CharField(
                choices=[
                    ("remain_on_study", "Participant to remain on trial"),
                    ("to_be_withdrawn", "Participant to be withdrawn from trial"),
                    (
                        "remain_on_study_modified",
                        "Patient remains on study but data analysis will be modified",
                    ),
                ],
                max_length=45,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="historicalprotocoldeviationviolation",
            name="action_required",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_protocol_incident.actionsrequired",
            ),
        ),
        migrations.AddField(
            model_name="protocoldeviationviolation",
            name="action_required",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_protocol_incident.actionsrequired",
            ),
        ),
    ]



from edc_adverse_event.navbars import ae_navbar_item, tmg_navbar_item
from edc_data_manager.navbar_item import dm_navbar_item
from edc_navbar import Navbar, NavbarItem, site_navbars
from edc_review_dashboard.navbars import navbar_item as review_navbar_item

no_url_namespace = False  # True if settings.APP_NAME == "meta_dashboard" else False

navbar = Navbar(name="meta_dashboard")


navbar.register(
    NavbarItem(
        name="screened_subject",
        title="Screening",
        label="Screening",
        fa_icon="fa-user-plus",
        codename="edc_screening.view_screening_listboard",
        url_name="screening_listboard_url",
        no_url_namespace=no_url_namespace,
    )
)

navbar.register(
    NavbarItem(
        name="consented_subject",
        title="Subjects",
        label="Subjects",
        fa_icon="fa-user-circle",
        codename="edc_subject_dashboard.view_subject_listboard",
        url_name="subject_listboard_url",
        no_url_namespace=no_url_namespace,
    )
)

navbar.register(review_navbar_item)
navbar.register(tmg_navbar_item)
navbar.register(ae_navbar_item)
navbar.register(dm_navbar_item)

site_navbars.register(navbar)



from django import forms
from edc_action_item.forms import ActionItemCrfFormMixin
from edc_constants.constants import NO, YES
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators import INVALID_ERROR

from ..models import UrinePregnancy


class UrinePregnancyFormValidator(CrfFormValidator):
    def clean(self):
        self.required_if(NO, field="performed", field_required="not_performed_reason")
        self.required_if(YES, field="performed", field_required="assay_date")
        if (
            self.cleaned_data.get("report_datetime")
            and self.cleaned_data.get("assay_date")
            and self.cleaned_data.get("assay_date")
            > self.cleaned_data.get("report_datetime").date()
        ):
            self.raise_validation_error(
                {"assay_date": "Cannot be after report datetime"}, INVALID_ERROR
            )
        self.applicable_if(YES, field="performed", field_applicable="bhcg_value")


class UrinePregnancyForm(CrfModelFormMixin, ActionItemCrfFormMixin, forms.ModelForm):
    form_validator_cls = UrinePregnancyFormValidator

    class Meta(ActionItemCrfFormMixin.Meta):
        model = UrinePregnancy
        fields = "__all__"



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_constants.choices import FASTING_CHOICES
from edc_constants.constants import EQ, FASTING
from edc_glucose.constants import GLUCOSE_HIGH_READING
from edc_lab.choices import RESULT_QUANTIFIER
from edc_lab_panel.model_mixin_factory import reportable_result_model_mixin_factory
from edc_model.models import BaseUuidModel
from edc_model.validators import datetime_not_future
from edc_reportable import (
    MILLIGRAMS_PER_DECILITER,
    MILLIMOLES_PER_LITER,
    MILLIMOLES_PER_LITER_DISPLAY,
)

from ..model_mixins import CrfModelMixin


class GlucoseModelMixin(
    reportable_result_model_mixin_factory(
        utest_id="glucose",
        verbose_name="Glucose",
        units_choices=(
            (MILLIGRAMS_PER_DECILITER, MILLIGRAMS_PER_DECILITER),
            (MILLIMOLES_PER_LITER, MILLIMOLES_PER_LITER_DISPLAY),
        ),
        decimal_places=2,
        validators=[MinValueValidator(1.00), MaxValueValidator(GLUCOSE_HIGH_READING)],
        exclude_attrs_for_reportable=True,
    ),
    models.Model,
):
    fasting = models.CharField(
        verbose_name="Was this fasting or non-fasting?",
        max_length=25,
        choices=FASTING_CHOICES,
        null=True,
        blank=False,
    )

    glucose_quantifier = models.CharField(
        max_length=10,
        choices=RESULT_QUANTIFIER,
        default=EQ,
    )

    def get_summary_options(self) -> dict:
        opts = super().get_summary_options()  # noqa
        fasting = True if self.fasting == FASTING else False
        opts.update(fasting=fasting)
        return opts

    class Meta:
        abstract = True


class GlucoseFbg(
    CrfModelMixin,
    GlucoseModelMixin,
    BaseUuidModel,
):

    assay_datetime = models.DateTimeField(
        verbose_name="Result date and time",
        validators=[datetime_not_future],
        null=True,
        blank=True,
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Glucose (FBG)"
        verbose_name_plural = "Glucose (FBG)"



# Generated by Django 4.2 on 2024-02-07 21:45

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0161_alter_birthoutcomes_options_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="followupexamination",
            name="abdominal_tenderness",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("not_evaluated", "Not evaluated")],
                max_length=25,
                verbose_name="Abdominal tenderness",
            ),
        ),
        migrations.AlterField(
            model_name="followupexamination",
            name="enlarged_liver",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("not_evaluated", "Not evaluated")],
                max_length=25,
                verbose_name="Enlarged liver",
            ),
        ),
        migrations.AlterField(
            model_name="followupexamination",
            name="jaundice",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("not_evaluated", "Not evaluated")],
                max_length=25,
                verbose_name="Jaundice",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfollowupexamination",
            name="abdominal_tenderness",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("not_evaluated", "Not evaluated")],
                max_length=25,
                verbose_name="Abdominal tenderness",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfollowupexamination",
            name="enlarged_liver",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("not_evaluated", "Not evaluated")],
                max_length=25,
                verbose_name="Enlarged liver",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfollowupexamination",
            name="jaundice",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("not_evaluated", "Not evaluated")],
                max_length=25,
                verbose_name="Jaundice",
            ),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-20 03:12

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("meta_screening", "0001_initial")]

    operations = [
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="already_fasted",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                max_length=15,
                verbose_name="Has the patient come to the clinic today already fasted?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="already_fasted",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                max_length=15,
                verbose_name="Has the patient come to the clinic today already fasted?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="already_fasted",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                max_length=15,
                verbose_name="Has the patient come to the clinic today already fasted?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="already_fasted",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                max_length=15,
                verbose_name="Has the patient come to the clinic today already fasted?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="already_fasted",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                max_length=15,
                verbose_name="Has the patient come to the clinic today already fasted?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="appt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="You may use today's date if the patient has already fasted and has agreed to complete the second stage of screening today",
                null=True,
                verbose_name="Appointment date for second stage of screening",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="appt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="You may use today's date if the patient has already fasted and has agreed to complete the second stage of screening today",
                null=True,
                verbose_name="Appointment date for second stage of screening",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="appt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="You may use today's date if the patient has already fasted and has agreed to complete the second stage of screening today",
                null=True,
                verbose_name="Appointment date for second stage of screening",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="appt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="You may use today's date if the patient has already fasted and has agreed to complete the second stage of screening today",
                null=True,
                verbose_name="Appointment date for second stage of screening",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="appt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="You may use today's date if the patient has already fasted and has agreed to complete the second stage of screening today",
                null=True,
                verbose_name="Appointment date for second stage of screening",
            ),
        ),
    ]



from copy import deepcopy

from django.db.utils import IntegrityError
from django.test import TestCase
from edc_constants.constants import FEMALE, NO, TBD, YES

from meta_edc.meta_version import PHASE_THREE, get_meta_version
from meta_screening.eligibility import EligibilityPartOne
from meta_screening.models import ScreeningPartOne

from ..options import get_part_one_eligible_options


class TestSubjectScreeningPartOneModel(TestCase):
    def test_eligibility_cls_eligible_yes(self):
        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        model_obj = ScreeningPartOne(**part_one_eligible_options)
        eligibility = EligibilityPartOne(model_obj=model_obj)
        # cls attrs
        self.assertEqual(eligibility.eligible, YES)
        self.assertDictEqual(eligibility.reasons_ineligible, {})
        # model attrs
        self.assertIsNone(getattr(model_obj, EligibilityPartOne.reasons_ineligible_fld_name))
        self.assertEqual(
            getattr(model_obj, EligibilityPartOne.eligible_fld_name),
            EligibilityPartOne.is_eligible_value,
        )

    def test_eligibility_cls_missing_eligible_tbd(self):
        """Assert missing data does not assess, eligible==TBD"""

        # try as cleaned data
        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        part_one_eligible_options.update(gender=None)
        eligibility = EligibilityPartOne(cleaned_data=part_one_eligible_options)
        self.assertEqual(eligibility.eligible, EligibilityPartOne.eligible_value_default)
        self.assertIn("gender", eligibility.reasons_ineligible)

        # try as model
        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        part_one_eligible_options.update(gender=None)
        model_obj = ScreeningPartOne(**part_one_eligible_options)
        eligibility = EligibilityPartOne(model_obj=model_obj)
        self.assertEqual(eligibility.eligible, TBD)
        self.assertIsNotNone(eligibility.reasons_ineligible)
        # model attrs
        self.assertIsNotNone(
            getattr(model_obj, EligibilityPartOne.reasons_ineligible_fld_name)
        )
        self.assertEqual(
            getattr(model_obj, EligibilityPartOne.eligible_fld_name),
            EligibilityPartOne.eligible_value_default,
        )

    def test_eligibility_cls_eligible_no(self):
        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        part_one_eligible_options.update(gender=FEMALE, pregnant=YES)
        model_obj = ScreeningPartOne(**part_one_eligible_options)
        eligibility = EligibilityPartOne(model_obj=model_obj)
        self.assertEqual(eligibility.eligible, NO)
        self.assertIn("pregnant", eligibility.reasons_ineligible)

    def test_defaults(self):
        opts = deepcopy(get_part_one_eligible_options())
        obj = ScreeningPartOne(**opts)
        obj.save()
        self.assertEqual(obj.eligible_part_one, YES)
        self.assertIsNone(obj.reasons_ineligible_part_one)
        self.assertFalse(obj.eligible)
        self.assertFalse(obj.consented)
        self.assertIsNotNone(obj.screening_identifier)

    def test_hospital_id_integrity(self):
        hospital_identifier = "111"
        obj = ScreeningPartOne(age_in_years=25, hospital_identifier=hospital_identifier)
        obj.save()
        obj = ScreeningPartOne(age_in_years=25, hospital_identifier=hospital_identifier)
        try:
            obj.save()
        except IntegrityError:
            pass
        else:
            self.fail("IntegrityError unexpectedly not raised.")

    def test_eligible(self):
        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        obj = ScreeningPartOne(**part_one_eligible_options)
        EligibilityPartOne(model_obj=obj)
        obj.save()
        self.assertIsNone(obj.reasons_ineligible_part_one)
        self.assertEqual(obj.eligible_part_one, YES)

    def test_ineligible(self):
        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        if get_meta_version() == PHASE_THREE:
            staying_nearby = "staying_nearby_12"
        else:
            staying_nearby = "staying_nearby_6"
        part_one_eligible_options.update({staying_nearby: NO})
        obj = ScreeningPartOne(**part_one_eligible_options)
        self.assertIsNone(obj.reasons_ineligible_part_one)
        obj.save()
        self.assertIn("Unable/Unwilling to stay nearby for", obj.reasons_ineligible_part_one)
        self.assertEqual(obj.eligible_part_one, NO)
        setattr(obj, staying_nearby, YES)
        obj.save()
        self.assertEqual(obj.eligible_part_one, YES)
        self.assertIsNone(obj.reasons_ineligible_part_one)



# Generated by Django 4.0.5 on 2022-06-30 08:40
from uuid import uuid4

from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations
from edc_action_item.identifiers import ActionIdentifier
from edc_constants.constants import CLOSED, HIGH_PRIORITY, NEW
from edc_visit_schedule.constants import OFFSCHEDULE_ACTION
from tqdm import tqdm


def create_missing_offschedule(apps, schema_editor):
    eos_model_cls = apps.get_model("meta_prn.endofstudy")
    offschedule_model_cls = apps.get_model("meta_prn.offschedule")
    action_item_model_cls = apps.get_model("edc_action_item.actionitem")
    action_type_model_cls = apps.get_model("edc_action_item.actiontype")

    subject_schedule_history_model_cls = apps.get_model(
        "edc_visit_schedule.subjectschedulehistory"
    )
    action_type = action_type_model_cls.objects.get(name=OFFSCHEDULE_ACTION)

    # update subject_schedule_history
    subject_schedule_history_model_cls.objects.filter(
        offschedule_model="meta_prn.endofstudy"
    ).update(offschedule_model="meta_prn.offschedule")

    sql = (
        "Select eos.id, eos.subject_identifier from meta_prn_endofstudy as eos "
        "left join meta_prn_offschedule as off on eos.subject_identifier = off.subject_identifier "
        "where off.id is null"
    )
    qs = eos_model_cls.objects.raw(sql)

    qs = [obj for obj in qs]
    total = len(qs)

    # create missing offschedule model instances
    for eos in tqdm(qs, total=total):
        # create action items model instances
        tracking_identifier = uuid4().hex
        try:
            action_item = action_item_model_cls.objects.get(
                subject_identifier=eos.subject_identifier, action_type=action_type
            )
        except ObjectDoesNotExist:
            action_item = action_item_model_cls(
                action_identifier=uuid4().hex.upper(),
                subject_identifier=eos.subject_identifier,
                action_type=action_type,
                report_datetime=eos.offschedule_datetime,
                priority=HIGH_PRIORITY,
                status=NEW,
                auto_created=True,
                reference_model="meta_prn.offschedule",
            )
            action_item.save()
            action_item.refresh_from_db()
            if not action_item.action_identifier:
                raise ValueError("Action identifier cannot be null")
        action_item.status = NEW
        action_item.save()
        action_item.refresh_from_db()
        offschedule = offschedule_model_cls(
            subject_identifier=eos.subject_identifier,
            action_identifier=action_item.action_identifier,
            tracking_identifier=tracking_identifier,
            action_item=action_item,
            report_datetime=eos.offschedule_datetime,
            offschedule_datetime=eos.offschedule_datetime,
            site_id=eos.site.id,
        )
        action_item.status = CLOSED
        action_item.save()
        offschedule.save()
        eos.parent_action_item = action_item
        eos.action_item.status = NEW
        eos.action_item.save()
        eos.action_item.refresh_from_db()
        eos.refresh_from_db()
        eos.save()

    for obj in eos_model_cls.objects.all():
        obj.save()


class Migration(migrations.Migration):
    dependencies = [
        ("meta_prn", "0034_auto_20220630_1110"),
    ]

    operations = [migrations.RunPython(create_missing_offschedule)]



import os  # noqa

from multisite import SiteID

from .defaults import *  # noqa

print(f"Settings file {__file__}")

# TZ Sites:
# SITE_ID = SiteID(default=20)  # Amana
SITE_ID = SiteID(default=10)  # Hindu Mandal
# SITE_ID = SiteID(default=40)  # Mwananyamala
# SITE_ID = SiteID(default=50)  # Mbagala
# SITE_ID = SiteID(default=60)  # Mnazi-Moja
# SITE_ID = SiteID(default=30)  # Temeke
INDEX_PAGE = "http://localhost:8000"
EDC_SITES_UAT_DOMAIN = False
DEBUG = True
ALLOWED_HOSTS = [
    "localhost",
    "mnazi-moja.tz.meta4.clinicedc.org",
    "mbagala.tz.meta4.clinicedc.org",
    "mwananyamala.tz.meta4.clinicedc.org",
    "hindu-mandal.tz.meta4.clinicedc.org",
    "temeke.tz.meta4.clinicedc.org",
    "amana.tz.meta4.clinicedc.org",
]

SECURE_SSL_REDIRECT = False
EDC_MODEL_ADMIN_CSS_THEME = "edc_purple"

if os.path.exists(BASE_DIR) and not os.path.exists(KEY_PATH):  # noqa
    os.makedirs(KEY_PATH)  # noqa
    AUTO_CREATE_KEYS = True



from .blood_results_fbc_admin import BloodResultsFbcAdmin
from .blood_results_hba1c_admin import BloodResultsHba1cAdmin
from .blood_results_ins_admin import BloodResultsInsAdmin
from .blood_results_lft_admin import BloodResultsLftAdmin
from .blood_results_lipid_admin import BloodResultsLipidAdmin
from .blood_results_rft_admin import BloodResultsRftAdmin



# Generated by Django 4.1 on 2022-08-21 18:54

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "meta_screening",
            "0055_alter_historicalscreeningpartone_creatinine_value_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="agree_to_p3",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not applicable, subject is not eligible based on the criteria above",
                    ),
                ],
                default="N/A",
                help_text="If patient is not continuing to the second stage today, advised to fast and provide a return appointment date below",
                max_length=15,
                verbose_name="Has the patient agreed to complete/return for the second stage of the screening?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="agree_to_p3",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not applicable, subject is not eligible based on the criteria above",
                    ),
                ],
                default="N/A",
                help_text="If patient is not continuing to the second stage today, advised to fast and provide a return appointment date below",
                max_length=15,
                verbose_name="Has the patient agreed to complete/return for the second stage of the screening?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="agree_to_p3",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not applicable, subject is not eligible based on the criteria above",
                    ),
                ],
                default="N/A",
                help_text="If patient is not continuing to the second stage today, advised to fast and provide a return appointment date below",
                max_length=15,
                verbose_name="Has the patient agreed to complete/return for the second stage of the screening?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="agree_to_p3",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not applicable, subject is not eligible based on the criteria above",
                    ),
                ],
                default="N/A",
                help_text="If patient is not continuing to the second stage today, advised to fast and provide a return appointment date below",
                max_length=15,
                verbose_name="Has the patient agreed to complete/return for the second stage of the screening?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="agree_to_p3",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not applicable, subject is not eligible based on the criteria above",
                    ),
                ],
                default="N/A",
                help_text="If patient is not continuing to the second stage today, advised to fast and provide a return appointment date below",
                max_length=15,
                verbose_name="Has the patient agreed to complete/return for the second stage of the screening?",
            ),
        ),
    ]



from django.apps import AppConfig as DjangoAppConfig
from django.conf import settings
from edc_facility.apps import AppConfig as BaseEdcFacilityAppConfig


class AppConfig(DjangoAppConfig):
    name = "meta_reports"
    verbose_name = "META Reports"


if settings.APP_NAME == "meta_reports":
    from dateutil.relativedelta import FR, MO, SA, SU, TH, TU, WE

    class EdcFacilityAppConfig(BaseEdcFacilityAppConfig):
        country = "tanzania"
        definitions = {
            "7-day-clinic": dict(
                days=[MO, TU, WE, TH, FR, SA, SU],
                slots=[100, 100, 100, 100, 100, 100, 100],
            ),
            "5-day-clinic": dict(days=[MO, TU, WE, TH, FR], slots=[100, 100, 100, 100, 100]),
        }



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils.translation import gettext as _
from edc_constants.choices import YES_NO
from edc_constants.constants import NOT_APPLICABLE
from edc_he.choices import RELATIONSHIP_CHOICES, STATUS
from edc_he.model_mixins import HouseholdModelMixin, income_model_mixin_factory
from edc_model.models import BaseUuidModel
from edc_model_fields.fields import OtherCharField

from ...model_mixins import CrfModelMixin


class HealthEconomicsUpdate(
    HouseholdModelMixin,
    income_model_mixin_factory(
        field_data={
            "avg_income": _(
                "Thinking over the last 12 months, can you tell me "
                "what the average earnings of the household have been?"
            ),
        }
    ),
    CrfModelMixin,
    BaseUuidModel,
):
    """A user model to learn about the household, wealth and
    opportunities in the community

    Introduced 22 FEB 2024
    """

    hoh = models.CharField(
        verbose_name=_("Are you the household head?"),
        max_length=15,
        choices=YES_NO,
        help_text=_(
            "By head of the household we mean the main decision "
            "maker in the household. The head can be either male or "
            "female. If two people are equal decision-makers, take "
            "the older person."
        ),
    )

    relationship_to_hoh = models.CharField(
        verbose_name=_("If No, what is your relationship to the household head?"),
        max_length=25,
        choices=RELATIONSHIP_CHOICES,
        default=NOT_APPLICABLE,
        help_text=_("Not applicable if patient is head of household"),
    )

    relationship_to_hoh_other = OtherCharField(
        verbose_name=_("If OTHER relationship, specify ..."),
    )

    rooms = models.IntegerField(
        verbose_name=_(
            "How many rooms does your dwelling have in total, without counting "
            "the bathrooms/ toilets or hallways/passageways?"
        ),
        validators=[MinValueValidator(1), MaxValueValidator(30)],
    )

    bedrooms = models.IntegerField(
        verbose_name=_("How many rooms are used for sleeping in your dwelling?"),
        validators=[MinValueValidator(0), MaxValueValidator(30)],
    )

    beds = models.IntegerField(
        verbose_name=_("How many beds does your dwelling have in total?"),
        validators=[MinValueValidator(0), MaxValueValidator(30)],
    )

    external_dependents = models.IntegerField(
        verbose_name=_(
            "Outside of this household, how many other people depend on this "
            "household's income?"
        ),
        validators=[MinValueValidator(0), MaxValueValidator(15)],
        help_text=_(
            "Insert '0' if no dependents other than the members in the household roster"
        ),
    )

    financial_status = models.CharField(
        verbose_name=_("Would you say your household's financial situation is?"),
        max_length=25,
        choices=STATUS,
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Health Economics: Baseline"
        verbose_name_plural = "Health Economics: Baseline"
